//+------------------------------------------------------------------+
//|                                             TickWriterBin_01.mq4 |
//|                                           maelstrom007@gmail.com |      
//|                                      Copyright  2009, painless  |
//|                            very loosely based on TickSave.mq4 by |
//|                                      Copyright  2006, komposter |
//|                                      mailto:komposterius@mail.ru |
//+------------------------------------------------------------------+
//ABSTRACT: To collect process and write tick data to binary files.
//The files selected must be separated by a single comma only.
//The destination directory is [metatrader home directory]\experts\files
//The record structure for the files is as follows.

//Date: Integer in the form YYYYMMDD - taken from system local.
//Time: Integer in the form of seconds since midnight - taken from system local.
//Bid: integer;
//Ask: integer;

//This is a beta version at best. It is a modified version of another's work
//and was not designed by me. The sole intent was to enable a quick solution
//to the problem of tick data storage.

//Data Format:
//Date: Storing the date in this form is not my ideal choice. I would prefer storing
//it as a daynumber from a constant epoch. Particularly for statistical performance
//measures.
//Time: I am satisfied with this solution.
//Bid & Ask: Storing as doubles involves more disk space and creates more overhead if
//used as doubles for analysis work. This is not a trivial matter. Loading years of tick
//data as doubles will consume a lot of memory and for no reason. Displaying the
//decimal point is a trivial matter and can be dealt with when required.

//To Do:
//Log Files: Seem to be working.
//Optionional rounding for 5 digit currencies.

// Syntax: All Globals start with an uppercase letter.
// Meaningful names are used because I am not a wanker.

extern string SymbolList = "AUDUSD,EURCHF,EURGBP,EURJPY,EURUSD,GBPJPY,GBPUSD,USDCAD,USDCHF,USDJPY";
extern int SleepFor = 500;             // sleep between writes 1000 = one second
extern bool AllowDuplicates = False;   // Allow duplicate ticks
extern bool IsRounded = True;          // Round to 4 digits?
extern bool ShowList = True;           // Show the symbol list on the screen?
extern bool DEBUG = False;           // Show the symbol list on the screen?

// Globals commence with g_
string g_SymbolsArray[32];
double g_PrevBid[32];
double g_PrevAsk[32];
double g_PrevRoundedAsk[32];
double g_PrevRoundedBid[32];
int g_FileHandle[32];
int g_SymbolsCount;
string g_LogFileName;

bool g_WasConnected;

//+------------------------------------------------------------------+

int init() {
   g_LogFileName = "TickWriter.log";
	if (!PrepareSymbolList()) {
      return(-1);
   }
   g_WasConnected = false;
   start();
   return(0);
}

//+------------------------------------------------------------------+

int start() {

	if (!ConnectionCheck()) {
      return(-1);
   }

	if (!OpenFiles()) {
      WriteToLogFile("Unable to open files for writing");
      return(-1);
   }

	while (!IsStopped()) {
		if (!IsConnected()) {
			g_WasConnected = false;
 			WriteToLogFile("Connection Lost");
		}
		else {
			if (!g_WasConnected) {
   			WriteToLogFile("Connection Established");
			}
			g_WasConnected = true;
			WriteTick();
		}
		Sleep(SleepFor);
	}

	CloseFiles();
	return(0);
}

//+------------------------------------------------------------------+

bool PrepareSymbolList() {
   int char;
   int len;
   int curSymbol;
   int pos;
	string symbolStr;
	bool isUnique;
	int numSymbols;
   
   // initialize
	g_SymbolsCount = 0;   // The Global
	ArrayResize(g_SymbolsArray, 32);
   char = 0;
   len = StringLen(SymbolList);
	symbolStr = "";
	
  
	for (pos = 0; pos <= len; pos++) {
		char = StringGetChar(SymbolList, pos);
		// Find the next symbol in the list
		if (char != ',' && pos != len) {
			symbolStr = symbolStr + CharToStr(char);
		}
		// Found a comma or the last char in the list
		else {
		   symbolStr = StringTrimLeft(symbolStr);
         symbolStr = StringTrimRight(symbolStr);
         
         if (!IsSymbolInList(symbolStr)) {
				return(false);
			}	
			else {
   			g_SymbolsArray[g_SymbolsCount] = symbolStr; // Print("Adding " + symbolStr + " to array at index " + g_SymbolsCount);
      		symbolStr = "";
            g_SymbolsCount++;
   		}	
      }
   } // for
   
   // check for duplicates   
   if (!AreSymbolsUnique()) {
      return(false);
   }
   
   // No more than 32 symbols
	if (g_SymbolsCount > 32) {
      Alert("No more than 32 symbols allowed!");
		return(false);
	}

	ArrayResize(g_SymbolsArray, g_SymbolsCount);
	ArrayResize(g_PrevBid, g_SymbolsCount);
	ArrayInitialize(g_PrevBid, -1);
	ArrayResize(g_PrevAsk, g_SymbolsCount);
	ArrayInitialize(g_PrevAsk, -1);
	ArrayResize(g_FileHandle, g_SymbolsCount);
	ArrayInitialize(g_FileHandle, -1);
	ArrayResize(g_PrevRoundedBid, g_SymbolsCount);
	ArrayInitialize(g_PrevRoundedBid, 0);
	ArrayResize(g_PrevRoundedAsk, g_SymbolsCount);
	ArrayInitialize(g_PrevRoundedAsk, 0);

	if (ShowList) {
	  Comment(SymbolList);
	}  
	return(true);
}

//+------------------------------------------------------------------+
//| Keep trying
//+------------------------------------------------------------------+
bool ConnectionCheck() {
   while (!IsConnected()) {
      Comment(AccountServer(), ": disconnected");
      if (IsStopped()) {
         return(false);
      }
		Sleep(100);
	}
	return(true);
}

//+------------------------------------------------------------------+

bool OpenFiles() {
	int _GetLastError;
	for (int curSymbol = 0; curSymbol < g_SymbolsCount; curSymbol++) {
		string FileName = StringConcatenate(g_SymbolsArray[curSymbol], ".fxh");
		g_FileHandle[curSymbol] = FileOpen( FileName, FILE_BIN|FILE_READ|FILE_WRITE );

		if (g_FileHandle[curSymbol] < 0 ) {
			_GetLastError = GetLastError();
			Alert( "FileOpen( " + FileName + ", FILE_READ | FILE_WRITE ) - Error #", _GetLastError );
			return(false);
		}

		if (!FileSeek(g_FileHandle[curSymbol], 0, SEEK_END)) {
			_GetLastError = GetLastError();
			Alert( "FileSeek( " + g_FileHandle[curSymbol] + ", 0, SEEK_END ) - Error #", _GetLastError );
			return(false);
		}

		g_PrevBid[curSymbol] = MarketInfo(g_SymbolsArray[curSymbol], MODE_BID);
		g_PrevAsk[curSymbol] = MarketInfo(g_SymbolsArray[curSymbol], MODE_ASK);
	}
	return(true);
}

//+------------------------------------------------------------------+
//| Wrire the binary files
//+------------------------------------------------------------------+
void WriteTick() {
	int _GetLastError;
	double curBid;
	double curAsk;
	int curDigits;
	int symbolIndex;
	string dateStr;
	int tmpDt;
	int tmpTm;
	string s;
	int bidAsInt, askAsInt;
	double roundedBid, roundedAsk;
	
	
	for (symbolIndex = 0; symbolIndex < g_SymbolsCount; symbolIndex++) {
		if (g_FileHandle[symbolIndex] < 0) {
         continue;
      }

		curBid = MarketInfo(g_SymbolsArray[symbolIndex], MODE_BID);
		curAsk = MarketInfo(g_SymbolsArray[symbolIndex], MODE_ASK);
		curDigits = MarketInfo(g_SymbolsArray[symbolIndex], MODE_DIGITS);

      // No change?
		if (NormalizeDouble(curBid - g_PrevBid[symbolIndex], curDigits) < 0.00000001 && NormalizeDouble(g_PrevBid[symbolIndex] - curBid, curDigits ) < 0.00000001) {
         continue;
      }
		if (NormalizeDouble(curAsk - g_PrevAsk[symbolIndex], curDigits) < 0.00000001 && NormalizeDouble(g_PrevAsk[symbolIndex] - curAsk, curDigits) < 0.00000001) {
         continue;
      }

      if (IsRounded) {
         roundedBid = round(g_PrevRoundedBid[symbolIndex], curBid, curDigits);
         roundedAsk = round(g_PrevRoundedAsk[symbolIndex], curAsk, curDigits);
         if (DEBUG) {
            Print("g_PrevRoundedBid = " + g_PrevRoundedBid[symbolIndex] + " curBid = " + curBid + " roundedBid = " + roundedBid);
         }   
         curBid = roundedBid;
         curAsk = roundedAsk;
         g_PrevRoundedBid[symbolIndex] = curBid;
         g_PrevRoundedAsk[symbolIndex] = curAsk;
      }
      
      // Allow duplicates
      if ((curBid != g_PrevBid[symbolIndex]) || (curAsk != g_PrevAsk[symbolIndex]) || (AllowDuplicates)) {
      
         if (!FileSeek(g_FileHandle[symbolIndex], 0, SEEK_END)) {
            _GetLastError = GetLastError();
            Alert( "FileSeek( " + g_FileHandle[symbolIndex] + ", 0, SEEK_END ) - Error #", _GetLastError );
			   continue;
         }
      
         // The local date		
         tmpDt = dateStrToInt(TimeToStr(TimeLocal(), TIME_DATE)); //Print("Date as int = " + tmp);
         // The local time		
         tmpTm = timeStrToSecs(TimeToStr(TimeLocal(), TIME_SECONDS));
         if (IsRounded) {
            bidAsInt = floatStrToInt(DoubleToStr(curBid, curDigits - 1));
            askAsInt = floatStrToInt(DoubleToStr(curAsk, curDigits - 1));
         }
         else {
            bidAsInt = floatStrToInt(DoubleToStr(curBid, curDigits));
            askAsInt = floatStrToInt(DoubleToStr(curAsk, curDigits));
         }
         FileWriteInteger(g_FileHandle[symbolIndex], tmpDt, LONG_VALUE);
         FileWriteInteger(g_FileHandle[symbolIndex], tmpTm, LONG_VALUE);
         FileWriteInteger(g_FileHandle[symbolIndex], bidAsInt, LONG_VALUE);
         FileWriteInteger(g_FileHandle[symbolIndex], askAsInt, LONG_VALUE);
         /*
         if (FileWrite(FileHandle[symbolIndex], TimeToStr( TimeCurrent(), TIME_DATE | TIME_SECONDS ), DoubleToStr( curBid, curDigits ), DoubleToStr(curAsk, curDigits)) < 0) {
            _GetLastError = GetLastError();
            Alert( "FileWrite() Error #", _GetLastError );
         }
         */
		} // allow duplicates or not
		g_PrevBid[symbolIndex] = curBid;
		g_PrevAsk[symbolIndex] = curAsk;
	}
} // WriteTick()

//+------------------------------------------------------------------+

void WriteToLogFile(string MsgStr) {
	int _GetLastError;
	int fileHandle;

	fileHandle = FileOpen(g_LogFileName, FILE_CSV|FILE_WRITE, ' ');
	if (fileHandle > 0) {
      if (!FileSeek(fileHandle, 0, SEEK_END)) {
         _GetLastError = GetLastError();
         Alert( "FileSeek( " + g_LogFileName + ", -strLEN, SEEK_END ) - Error #", _GetLastError );
      }
      else {
         if (FileWrite(fileHandle,  TimeToStr(TimeLocal(), TIME_DATE|TIME_SECONDS) + "  " + MsgStr) < 0) {
            _GetLastError = GetLastError();
            Alert("FileWrite() Error #", _GetLastError);
         }
      }
      FileFlush(fileHandle);
      FileClose(fileHandle);
	}
} // WriteToLogFile

//+------------------------------------------------------------------+

void CloseFiles() {
	for (int curSymbol = 0; curSymbol < g_SymbolsCount; curSymbol++) {
		if (g_FileHandle[curSymbol] > 0) {
			FileClose(g_FileHandle[curSymbol]);
			g_FileHandle[curSymbol] = -1;
		}
	}
}

//+------------------------------------------------------------------+

int dateStrToInt(string theDateStr) {
int d;
	d = StrToInteger(StringConcatenate(StringSubstr(theDateStr, 0, 4), StringSubstr(theDateStr, 5, 2), StringSubstr(theDateStr, 8, 2)));
	return(d);
}

//+------------------------------------------------------------------+

int timeStrToSecs(string theTimeStr) {
int h, m, s;
   h = StrToInteger(StringSubstr(theTimeStr, 0, 2)) * 3600;
   m = StrToInteger(StringSubstr(theTimeStr, 3, 2)) * 60;
   s = StrToInteger(StringSubstr(theTimeStr, 6, 2));
   return(h + m + s);
}

//+------------------------------------------------------------------+

int floatStrToInt(string theFloat) {
int p;
string s;
   p = StringFind(theFloat, ".", 0); 
   s = StringSubstr(theFloat, 0, p) + StringSubstr(theFloat, p + 1); 
   return(StrToInteger(s));   
}

//+------------------------------------------------------------------+
// Confirm that the user has this file in their open list
bool IsSymbolInList(string theSymbol) {
   bool found;
   
   found = true;
   MarketInfo(theSymbol, MODE_BID);
	if (GetLastError() == 4106) {
	  Alert( "This symbol does not exist ", theSymbol, "!!!" );
	  found = false;
	}  
	return(found);
}

//+------------------------------------------------------------------+
// No duplicate symbols
bool AreSymbolsUnique() {
   int i;
   int indexToSearch;
   bool isUnique = true;
   
   for (indexToSearch = 0; indexToSearch < g_SymbolsCount; indexToSearch++) {
      for (i = 0; i < g_SymbolsCount; i++) {
         if ((indexToSearch != i) && (g_SymbolsArray[i] == g_SymbolsArray[indexToSearch])) {
            isUnique = false;
            Alert("You have two " + g_SymbolsArray[i] + " in your list");
         }   
      }
   }
   return(isUnique);
}   

//+------------------------------------------------------------------+
// The rounding function
double round(double prevPrice, double currentPrice, int digits) {
   double result = 0;
   
   currentPrice *= MathPow(10, digits - 1);
   prevPrice  *= MathPow(10, digits - 1); // Print("currentPrice = " + currentPrice + " prevPrice = " + prevPrice);
   if (MathFloor(currentPrice) > prevPrice) {
      result = MathFloor(currentPrice);               
   }
   else if (MathCeil(currentPrice) < prevPrice) {
      result = MathCeil(currentPrice);
   }
   else {
      result = prevPrice;
   }
   result /= MathPow(10, digits - 1);   
   return(result);
}


